@@ -1,19 +1,31 @@
 import bitstruct
 
-from typing import List, Dict, Callable, Union, Any, Tuple
-from collections import OrderedDict
+from typing import List, Dict, Union, Any, Tuple
 
-from ldfparser.encoding import LinSignalType
+from .encoding import LinSignalType
 
 class LinSignal:
 
-	def __init__(self, name: str, width: int, init_value: List[int]):
+	def __init__(self, name: str, width: int, init_value: Union[int, List[int]]):
 		self.name = name
 		self.width = width
 		self.init_value = init_value
 
-	def __str__(self):
-		return "Signal(name=" + self.name + ")"
+	def is_array(self):
+		return isinstance(self.init_value, List)
+
+	@staticmethod
+	def create(name: str, width: int, init_value: Union[int, List[int]]):
+		if isinstance(init_value, List):
+			if width % 8 != 0:
+				raise ValueError(f"array signal {name}:{width} must be a multiple of 8 long (8, 16, 24, ..)")
+			if width < 8 or width > 64:
+				raise ValueError(f"array signal {name}:{width} must be 8-64bits long")
+			if len(init_value) != width / 8:
+				raise ValueError(f"array signal {name}:{width} init value is invalid {len(init_value)}")
+		if isinstance(init_value, int) and (width < 1 or width > 16):
+			raise ValueError(f"scalar signal {name}:{width} must be 1-16bits long")
+		return LinSignal(name, width, init_value)
 
 class LinFrame:
 
@@ -34,16 +46,21 @@ class LinFrame:
 				raise ValueError(str(self) + ":" + str(signal[1]) + " overlapping")
 			if signal[0] != offset:
 				padding = signal[0] - offset
-				pattern += "p" + str(padding)
+				pattern += f"p{padding}"
 				offset += padding
 			if offset + signal[1].width > frame_bits:
-				raise ValueError(str(self) + ":" + str(signal[1]) + " out of frame")
-			pattern += "u" + str(signal[1].width)
+				raise ValueError(f"{signal[1]} with offset {signal[0]} spans outside {self}")
+			if signal[1].is_array():
+				pattern += f"u8" * int(signal[1].width / 8)
+			else:
+				pattern += f"u{signal[1].width}"
 			offset += signal[1].width
 		if offset < frame_bits:
-			pattern += "p" + str(frame_bits - offset)
+			pattern += f"p{frame_bits - offset}"
 		return pattern
 
+	def _get_signal(self, name: str):
+		return next((x for x in self.signals if x.name == name), None)
 
 	def raw(self, data: Dict[str, int]) -> bytearray:
 		"""
@@ -52,9 +69,15 @@ class LinFrame:
 		message = []
 		for signal in self.signals:
 			if data.get(signal.name):
-				message.append(data.get(signal.name))
+				if signal.is_array():
+					message += data.get(signal.name)
+				else:
+					message.append(data.get(signal.name))
 			else:
-				message.append(signal.init_value[0])
+				if signal.is_array():
+					message += signal.init_value
+				else:
+					message.append(signal.init_value)
 		return self._flip_bytearray(self._packer.pack(*message))
 
 	def data(self, data: Dict[str, Union[str, int, float]], converters: Dict[str, LinSignalType]) -> bytearray:
@@ -65,7 +88,7 @@ class LinFrame:
 		for value in data.items():
 			if converters[value[0]] is None:
 				raise ValueError('No encoder found for ' + value[0])
-			converted[value[0]] = converters[value[0]].encode(value[1])
+			converted[value[0]] = converters[value[0]].encode(value[1], self._get_signal(value[0]))
 		return self.raw(converted)
 
 	def parse_raw(self, data: bytearray) -> Dict[str, int]:
@@ -74,8 +97,17 @@ class LinFrame:
 		"""
 		message = {}
 		unpacked = self._packer.unpack(self._flip_bytearray(data))
-		for i in range(len(unpacked)):
-			message[self.signals[i].name] = unpacked[i]
+		signal_index = 0
+		i = 0
+		while i < len(unpacked):
+			if self.signals[signal_index].is_array():
+				signal_size = int(self.signals[signal_index].width / 8)
+				message[self.signals[signal_index].name] = list(unpacked[i:i+signal_size])
+				i += signal_size - 1
+			else:
+				message[self.signals[signal_index].name] = unpacked[i]
+			signal_index += 1
+			i += 1
 		return message
 
 	def parse(self, data: bytearray, converters: Dict[str, LinSignalType]) -> Dict[str, Union[str, int, float]]:
@@ -87,21 +119,9 @@ class LinFrame:
 		for value in tmp.items():
 			if converters[value[0]] is None:
 				raise ValueError('No decoder found for ' + value[0])
-			output[value[0]] = converters[value[0]].decode(value[1])
+			output[value[0]] = converters[value[0]].decode(value[1], self._get_signal(value[0]))
 		return output
 
-	def compare(self, previous: bytearray, current: bytearray) -> Dict[str, int]:
-		"""
-		Returns a mapping between Signal names and their values, the result will
-		only contain signals that have changed compared to the previous message
-		"""
-		prev = self.parse(previous)
-		curr = self.parse(current)
-		raise NotImplementedError()
-
-	def __str__(self):
-		return "Frame(id=" + str(self.frame_id) + ", name=" + self.name + ")"
-
 	def _flip_bytearray(self, data: bytearray) -> bytearray:
 		flipped = bytearray()
 		for i in data:
